import { Random } from '../../random/generator/Random';
import { Arbitrary } from './definition/Arbitrary';
import { Shrinkable } from './definition/Shrinkable';

/** @hidden */
class OneOfArbitrary<T> extends Arbitrary<T> {
  readonly idArb: Arbitrary<number>;
  constructor(readonly arbs: Arbitrary<T>[]) {
    super();
  }
  generate(mrng: Random): Shrinkable<T> {
    const id = mrng.nextInt(0, this.arbs.length - 1);
    return this.arbs[id].generate(mrng);
  }
  withBias(freq: number) {
    return new OneOfArbitrary(this.arbs.map(a => a.withBias(freq)));
  }
}

/**
 * For one of the values generated by `...arbs` - with all `...arbs` equiprobable
 *
 * **WARNING**: It expects at least one arbitrary
 *
 * @param arbs Arbitraries that might be called to produce a value
 */
function oneof<T>(...arbs: Arbitrary<T>[]): Arbitrary<T> {
  if (arbs.length === 0) {
    throw new Error('fc.oneof expects at least one parameter');
  }
  return new OneOfArbitrary([...arbs]);
}

export { oneof };
